
/*
 * Requires support for Javascript file API and progress indicators.
 *
 * Code by Brian Hayes, Feb 2013. Please feel free to
 * adopt or adapt anything here you find useful.
 */





function drivel() {

	// drivelStatus = empty, running, paused, reset;
	// sourceStatus = empty, loading, loaded

	var drivelStatus = "empty";
	var sourceStatus = "empty"

	var sourceTextString = "";
	var drivelTextString = "";
	var fileOrUrlString = "";
	
	var recursionLevel = 0;
	
	var drivelTextScroller = document.getElementById("drivel-scroller");
	var sourceTextDisplayBox = document.getElementById("source-text");
	var drivelTextDisplayBox = document.getElementById("drivel-text");
	
	var stopGoButton = document.getElementById("stop-go");
	stopGoButton.onclick = function() { handleEvent(this, "click") };
	
	var clearDrivelButton = document.getElementById("clear-drivel");
	clearDrivelButton.onclick = function() { handleEvent(this, "click") };
	
	var clearSourceButton = document.getElementById("clear-source");
	clearSourceButton.onclick = function() { handleEvent(this, "click") };
	
	var recurseButton = document.getElementById("recurse");
	recurseButton.onclick = function() { handleEvent(this, "click") };
	
	var drivelOrderSelector = document.getElementById("drivel-order-select");
	drivelOrderSelector.onchange = function() { handleEvent(this, "change") };
	
	var ajaxSelector = document.getElementById("ajax-source-select");
	ajaxSelector.onchange = function() { handleEvent(this, "change") };
		
	var fileChooser = document.getElementById("file-choose");
	fileChooser.onchange = function() { handleEvent(this, "change") };
	
	var fastMode = document.getElementById("fast-mode");
	
	var faqButton = document.getElementById("faq-button");
	faqButton.onclick = showHideFAQ;
	
	
	// one function to deal with all the enabling and disabling
	// of UI elements depending on the state of the program
	// We don't want people clobbering the source text while
	// we're using it to generate drivel
	
	function updateUI() {
		if ( drivelStatus === "empty" && sourceStatus === "empty" ) {
			stopGoButton.innerHTML = "Go";
			ajaxSelector.selectedIndex = 0;
			disableUI([clearDrivelButton, clearSourceButton, stopGoButton, recurseButton]);
			enableUI([drivelOrderSelector, fastMode, ajaxSelector, fileChooser]);
		}
		else if ( drivelStatus === "empty" && sourceStatus === "loading" ) {
			stopGoButton.innerHTML = "Go";
			disableUI([clearDrivelButton, clearSourceButton, stopGoButton, ajaxSelector, fileChooser, recurseButton]);
			enableUI([drivelOrderSelector, fastMode]);
		}
		else if ( drivelStatus === "empty" && sourceStatus === "loaded" ) {
			stopGoButton.innerHTML = "Go";
			disableUI([clearDrivelButton, recurseButton]);
			enableUI([stopGoButton, drivelOrderSelector, fastMode, clearSourceButton, ajaxSelector, fileChooser]);
		}
		else if ( drivelStatus === "running" && sourceStatus === "loaded" ) {
			stopGoButton.innerHTML = "Stop";
			disableUI([drivelOrderSelector, fastMode, clearSourceButton, ajaxSelector, fileChooser, recurseButton]);
			enableUI([stopGoButton, clearDrivelButton]);
		}
		else if ( drivelStatus === "paused" && sourceStatus === "empty" ) {
			stopGoButton.innerHTML = "Go";
			disableUI([stopGoButton, clearSourceButton]);
			enableUI([clearDrivelButton, drivelOrderSelector, fastMode, ajaxSelector, fileChooser, recurseButton]);
		}
		else if ( drivelStatus === "paused" && sourceStatus === "loading" ) {
			stopGoButton.innerHTML = "Go";
			disableUI([stopGoButton, clearSourceButton, ajaxSelector, fileChooser, recurseButton]);
			enableUI([clearDrivelButton, drivelOrderSelector, fastMode]);
		}
		else if ( drivelStatus === "paused" && sourceStatus === "loaded" ) {
			stopGoButton.innerHTML = "More";
			enableUI([stopGoButton, clearDrivelButton, drivelOrderSelector, fastMode, clearSourceButton, ajaxSelector, fileChooser, recurseButton]);
		}
		else if ( drivelStatus === "reset" && sourceStatus === "empty" ) {
			stopGoButton.innerHTML = "Go";
			disableUI([stopGoButton, clearSourceButton]);
			enableUI([clearDrivelButton, drivelOrderSelector, fastMode, ajaxSelector, fileChooser, recurseButton]);
		}
		else if ( drivelStatus === "reset" && sourceStatus === "loading" ) {
			stopGoButton.innerHTML = "Go";
			disableUI([stopGoButton, clearSourceButton, ajaxSelector, fileChooser, recurseButton]);
			enableUI([clearDrivelButton, drivelOrderSelector, fastMode]);
		}
		else if ( drivelStatus === "reset" && sourceStatus === "loaded" ) {
			stopGoButton.innerHTML = "Go";
			enableUI([stopGoButton, clearDrivelButton, drivelOrderSelector, fastMode, clearSourceButton, ajaxSelector, fileChooser, recurseButton]);
		}
	}
	
	function disableUI(uiArray) {
		for ( var i = 0 ; i < uiArray.length ; i++ ) {
			var elt = uiArray[i];
			if ( !elt.disabled ) {
				elt.disabled = "disabled";
			}
		}
	}
	
	function enableUI(uiArray) {
		for ( var i = 0 ; i < uiArray.length ; i++ ) {
			var elt = uiArray[i];
			if ( elt.disabled ) {
				elt.removeAttribute("disabled");
			}
		}
	}
	
	
	// centralized engine for dealing with clicks, selects and other events
	// Note that the 'evt' argument to this function is never actually used;
	// it turns out that just known where the event is coming from (the 'elt'
	// argument) is enough 
	
	function handleEvent(elt, evt) {

		if (elt === stopGoButton) {
			if (drivelStatus === "running") {
				drivelStop();
				drivelStatus = "paused";
			}
			else if (sourceStatus === "loaded" ) {
				if (drivelStatus === "reset") {
					drivelEndParagraph();
				}
				if (drivelStatus !== "paused" ) {
					drivelInit();
					drivelNewPattern();
					drivelBanner();
					drivelNewParagraph();			
				}
				drivelStatus = "running";
				drivelGo();
			}
			updateUI();
		}
		else if (elt === recurseButton) {
			if (drivelTextString.length > 0) {
				sourceTextString = drivelTextString;
				sourceTextDisplayBox.innerHTML = drivelTextString;
				drivelTextDisplayBox.innerHTML = "";
				drivelStatus = "empty";
				recursionLevel++;
				clearFileChooser();
				clearAjaxSelector();
				updateUI();
			}
		}
		else if (elt === clearDrivelButton) {
			drivelStop();
			drivelInit();
			drivelTextDisplayBox.innerHTML = "";
			drivelStatus = "empty";
			updateUI();
		}
		else if (elt === clearSourceButton) {
			drivelStop();
			drivelInit();
			sourceTextString = "";
			sourceTextDisplayBox.innerHTML = "";
			if ( drivelStatus !== "empty" ) {
				drivelStatus = "reset";
			}
			sourceStatus = "empty";
			recursionLevel = 0;
			clearFileChooser();
			clearAjaxSelector();
			updateUI();
		}
		else if (elt === drivelOrderSelector) {
			drivelOrder = elt.selectedIndex;
			drivelInit();
			drivelNewPattern();
			if ( drivelStatus !== "empty" ) {
				drivelStatus = "reset";
			}
			updateUI();
		}
		else if (elt === ajaxSelector) {
			if ( elt.selectedIndex === 0 ) {
				handleEvent(clearSourceButton, null);
				return; 
			}
			var url = elt.options[elt.selectedIndex].value;
			if ( url ) {
				sourceStatus = "loading";
				sourceTextDisplayBox.innerHTML = "";
				fileOrUrlString = url;
				if (drivelStatus === "paused") {
					drivelInit();
					drivelNewPattern();
					drivelStatus = "reset";
				}
				updateUI();
				startAjax(url);
				clearFileChooser();
				recursionLevel = 0;
			}
		}
		else if (elt === fileChooser) {
			var f = elt.files[0];
			if ( f ) {
				sourceStatus = "loading";
				sourceTextDisplayBox.innerHTML = "";
				fileOrUrlString = f.name;
				if (drivelStatus === "paused") {
					drivelInit();
					drivelNewPattern();
					drivelStatus = "reset";
				}
				updateUI();
				startFileGet(f);
				clearAjaxSelector();
				recursionLevel = 0;
			}
			else {
				handleEvent(clearSourceButton, null);
			}
		}
	}
	
	
	
	// The attributes of the files object are programmatically immutable,
	// so the only way to clear them from the UI is to replace the whole input element
	// with a new one. See http://stackoverflow.com/questions/829571/clearing-an-html-file-upload-field-via-js/829740#829740
	// and http://aspnetupload.com/Clear-HTML-File-Input.aspx
	
	function clearFileChooser() {
		var oldChooser = fileChooser;
		var newChooser = document.createElement("input");
		newChooser.type = "file";
		newChooser.accept = oldChooser.accept;
		newChooser.id = oldChooser.id;
		oldChooser.parentNode.replaceChild(newChooser, oldChooser);
		fileChooser = document.getElementById(newChooser.id);
		fileChooser.onchange = function() { handleEvent(this, "change") };
	}
	
	
	function clearAjaxSelector() {
		ajaxSelector.selectedIndex = 0;
	}
	
	function showHideFAQ() {
		var faqDiv = document.getElementById("faq");
		if (faqDiv.style.display === "none") {
			faqDiv.style.display = "block";
			faqButton.innerHTML = "Hide FAQ";
		}
		else {
			faqDiv.style.display = "none";
			faqButton.innerHTML = "Show FAQ";
		}
	}
	
	
	
	
	// finally we get down to the actual business of generating drivel
	
	var drivelOrder = 3;
	var maxSourceIndex = 0;
	var maxDrivelChars = 1000;
	var drivelChars = 0;
	var pattern = "";
	var drivelTimer = null;
	var scrollTimer = null;
	var drivelParagraph = null;
	
	
	// reset to a known initial state
	
	function drivelInit() {
		pattern = "";
		drivelTextString = "";
		drivelChars = 0;
		drivelTimer = null;
		scrollTimer = null;
	}
	
	
	// to get the process started, we set the pattern to a random
	// block of characters from the source text, of length = drivelOrder
	
	function drivelNewPattern() {
		maxSourceIndex = sourceTextString.length - drivelOrder - 1;     // -1 to make sure we don't run off the end
		var idx = Math.floor(Math.random() * maxSourceIndex);
		pattern = sourceTextString.substring(idx, idx + drivelOrder);
	}
	
	
	// When the user chooses a new source text or changes the order of the
	// drivel, we start a new paragraph of drivel
	
	function drivelNewParagraph() {
		drivelParagraph = document.createElement("p");
		drivelTextDisplayBox.appendChild(drivelParagraph);
	}
	
	
	// at the end of each paragraph, draw a horizontal rule
	
	function drivelEndParagraph() {
		if (drivelParagraph) {
			drivelTextDisplayBox.appendChild(document.createElement("hr"));
			drivelParagraph = null;
		}
	}
	
	
	// at the beginning of each paragraph, a banner identifying the
	// source text and order
	
	function drivelBanner() {
		var bannerStr = "<p class='banner'>Order " + drivelOrder + " drivel based on " + fileOrUrlString;
		if (recursionLevel > 0) {
			bannerStr += "  :  Recursion level " + recursionLevel;
		}
		bannerStr += "</p>";
		drivelTextDisplayBox.innerHTML += bannerStr;
	}
	

	// What happens when the "Go" button is clicked. In fast mode,
	// just generate a set of 1,000 drivel characters in a loop, appending
	// to the output one by one. If we're not in fast mode, start timers
	// to throttle the process of driveling and also the scrolling of
	// the drivel panel.

	function drivelGo() {
		if ( fastMode.checked ) {
			for (var i = 0 ; i < maxDrivelChars ; i++) {
				var c = drivelNextChar();
				if (c === "") {									// failed to generate a character, so quit
					drivelStop();
					drivelEndParagraph();
					drivelChars = 0;
					drivelStatus = "reset";
					updateUI();
					break;
				}
				else {
					drivelParagraph.innerHTML += c;
					drivelTextString += c;
					pattern = pattern.substring(1) + c;     // ab + c --> bc
				}
			}
			drivelTextScroller.scrollTop += 1000;			// see http://www.adp-gmbh.ch/web/js/scroll.html
			if (drivelStatus ===  "running") {
				drivelStatus = "paused";
			}
			updateUI();
		}
		else {
			drivelTimer = setInterval(function() { drivelWriteChar(); }, 50);
			scrollTimer = setInterval(function() { drivelTextScroller.scrollTop += 1 }, 10);
		}
	}
	
	
	// only makes sense to call this if status="running"

	function drivelStop() {
		if (drivelTimer) { clearInterval(drivelTimer); }
		if (scrollTimer) { clearInterval(scrollTimer); }
	}
	
	
	// This is where the drivel comes from. 
	  
	function drivelNextChar() {
    var start = Math.floor(Math.random() * maxSourceIndex);				// random point in text
    
    if ( drivelOrder === 0 ) {
	    return(sourceTextString.charAt(start));
    }
    else {
	    var idx = sourceTextString.indexOf(pattern, start);					// first occurrence of pattern after start point
	    if ( idx === -1 || idx >= maxSourceIndex) {																					// pattern not found
		    idx = sourceTextString.indexOf(pattern, 0);								// so try again from start of string
	    }
	    if ( idx === -1 || idx >= maxSourceIndex ) {								// still not found ; return empty string as sentinel
	    	return("");
	    }
	    return(sourceTextString.charAt(idx + drivelOrder));					// get char following pattern
    }
	}
	
	
	// called every 50ms by the drivel timer
	
	function drivelWriteChar() {
		if ( drivelChars >= maxDrivelChars ) {				// reached the limit; we're done for now
			drivelStop();
			drivelChars = 0;
			drivelStatus = "paused";
			updateUI();
		}
		else {
		  var c = drivelNextChar();
		  if (c === "") {                     // failed to find a char, so shut down and close paragraph
				drivelStop();
				drivelEndParagraph();
				drivelChars = 0;
				drivelStatus = "reset";
				updateUI();
		  }
		  else {
			  drivelChars++;
			  drivelParagraph.innerHTML += c;							// append to the output element
			  drivelTextString += c;											// and the output string
			  pattern = pattern.substring(1) + c;					// and to the pattern
			}
	  }
	}
  
  
  
  // Read from the local file system.
	
	function startFileGet(f) {											// caller hands us a file handle

	  if ( Modernizr.filereader ) {									// only if Modernizr says the browser can do this
	
		  var reader = new FileReader();							// Create Filereader object
		  
		  reader.onload = function(evt) {							// add handler for Filereader, invoked on successdul read
			  sourceTextString = evt.target.result;			// get plain text
				sourceTextString = sourceTextString.toString();     // convert to js string type (shouldn't be needed?)
			  if (f.type === "text/plain") {
				  sourceTextString = collapseWhiteSpace(sourceTextString);   // remove line breaks
				}
			  drivelNewPattern();
			  drivelInit();
			  sourceTextDisplayBox.innerHTML = sourceTextString;
			  sourceStatus = "loaded";
			  updateUI();
		  }
		}
   reader.readAsText(f);   		// ask our new Filereader to get the text
  }
	

  // Fetch sample documents from the server, when the user selects.

	function startAjax(url) {
    var xhr = new XMLHttpRequest();
    xhr.onreadystatechange = function() {
			if (xhr.readyState == 4 && xhr.status == 200) {
				sourceTextString = xhr.responseText;
				sourceTextString = collapseWhiteSpace(sourceTextString);
			  drivelNewPattern();
			  drivelInit();
			  sourceTextDisplayBox.innerHTML = sourceTextString;
			  sourceStatus = "loaded";
			  updateUI();
			}
			else if (xhr.readyState == 4) {
			  sourceStatus = "empty";
			  updateUI();				
				alert("Failed to fetch file. Sorry.");
			}
		}
		xhr.open("GET", url);
		xhr.send(null);
  }






  // Newlines, returns and tabs all become spaces. Then
  // runs of multiple consecutive spaces are replaced with
  // a single space.

  function collapseWhiteSpace(text) {
  	var newText = text.replace(/[\n\r\t]/g," ");
  	newText = newText.replace(/\ \ +/g," ");
  	return(newText);
  }
  
  
  // Unless Modernizr says okay, hide the local-file-select UI.
  
  if ( ! Modernizr.filereader ) {
	  var fileUI = document.getElementById("local-file");
	  fileUI.setAttribute("style", "display: none");
  }
  
  
  
  
  // stuff to do on first loading
  
  clearFileChooser();			// needed for Firefox, which otherwise doesn't reset UI elements on reload
  clearAjaxSelector();
  drivelOrderSelector.selectedIndex = 3;   // again, FF uses cached state
  
  ajaxSelector.selectedIndex = 1;			// late-in-the-day decision to load a text by default
  handleEvent(ajaxSelector, null);		// get the Onegin text
  
  document.getElementById("faq").style.display = "none";
  
  updateUI();
  
}

drivel();    // Just do it.

